"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnnotatedConfigModel = exports.OutputFormat = void 0;
const js_yaml_1 = require("js-yaml");
const StylingUtil_1 = require("../utils/StylingUtil");
const messages_1 = require("../messages");
const node_path_1 = __importDefault(require("node:path"));
var OutputFormat;
(function (OutputFormat) {
    OutputFormat["RAW_YAML"] = "RAW_YAML";
    OutputFormat["STYLED_YAML"] = "STYLED_YAML";
})(OutputFormat || (exports.OutputFormat = OutputFormat = {}));
class AnnotatedConfigModel {
    codeAnalyzer;
    userRules;
    allDefaultRules;
    // Note that it is important that we calculate the relevant engines list based on (the user rule selection with no
    // config) plus (user rule selection with user config) since we still want to show the "disable_engine" config value
    // in the output if a user even if selects an engine that is currently disabled. But we don't want to the engine
    // configs not associated with the user's rule selection, thus we can't use the engines from allDefaultRules.
    relevantEngines;
    constructor(codeAnalyzer, userRules, allDefaultRules, relevantEngines) {
        this.codeAnalyzer = codeAnalyzer;
        this.userRules = userRules;
        this.allDefaultRules = allDefaultRules;
        this.relevantEngines = relevantEngines;
    }
    toFormattedOutput(format) {
        // istanbul ignore else: Should be impossible
        if (format === OutputFormat.STYLED_YAML) {
            return new StyledYamlFormatter(this.codeAnalyzer, this.userRules, this.allDefaultRules, this.relevantEngines).toYaml();
        }
        else if (format === OutputFormat.RAW_YAML) {
            return new PlainYamlFormatter(this.codeAnalyzer, this.userRules, this.allDefaultRules, this.relevantEngines).toYaml();
        }
        else {
            throw new Error(`Unsupported`);
        }
    }
}
exports.AnnotatedConfigModel = AnnotatedConfigModel;
class YamlFormatter {
    config;
    codeAnalyzer;
    userRules;
    allDefaultRules;
    relevantEngines;
    constructor(codeAnalyzer, userRules, allDefaultRules, relevantEngines) {
        this.config = codeAnalyzer.getConfig();
        this.codeAnalyzer = codeAnalyzer;
        this.userRules = userRules;
        this.allDefaultRules = allDefaultRules;
        this.relevantEngines = relevantEngines;
    }
    toYamlSectionHeadingComment(commentText) {
        const horizontalLine = '='.repeat(70);
        return this.toYamlComment(`${horizontalLine}\n${commentText}\n${horizontalLine}`);
    }
    toYamlUncheckedField(fieldName, fieldValue) {
        return (0, js_yaml_1.dump)({ [fieldName]: fieldValue }).trim();
    }
    toYamlUncheckedFieldWithInlineComment(fieldName, fieldValue, commentText) {
        const yamlCode = this.toYamlUncheckedField(fieldName, fieldValue);
        const comment = this.toYamlComment(commentText);
        return yamlCode.replace(/(\r?\n|$)/, ` ${comment}$1`);
    }
    toYamlFieldWithFieldDescription(fieldName, resolvedValue, fieldDescription) {
        const resolvedValueJson = JSON.stringify(resolvedValue);
        const defaultValueJson = JSON.stringify(fieldDescription.defaultValue);
        let yamlField;
        if (!fieldDescription.wasSuppliedByUser && resolvedValueJson !== defaultValueJson) {
            // Whenever the user did not supply the value themselves but the resolved value is different from the
            // default value, this means the value was not a "fixed" value but a value "calculated" at runtime.
            // Since "calculated" values often depend on the specific environment, we do not want to actually hard code
            // this value into the config since checking in the config to CI/CD system may create a different value.
            const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.last-calculated-as', [resolvedValueJson]);
            yamlField = this.toYamlUncheckedFieldWithInlineComment(fieldName, fieldDescription.defaultValue, commentText);
        }
        else {
            yamlField = this.toYamlField(fieldName, resolvedValue, fieldDescription.defaultValue);
        }
        return this.toYamlComment(fieldDescription.descriptionText) + "\n" + yamlField;
    }
    toYamlField(fieldName, resolvedValue, defaultValue) {
        const resolvedValueJson = JSON.stringify(resolvedValue);
        const defaultValueJson = JSON.stringify(defaultValue);
        if (resolvedValueJson === defaultValueJson) {
            return this.toYamlUncheckedField(fieldName, resolvedValue);
        }
        const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.modified-from', [defaultValueJson]);
        resolvedValue = replaceAbsolutePathsWithRelativePathsWherePossible(resolvedValue, this.config.getConfigRoot() + node_path_1.default.sep);
        return this.toYamlUncheckedFieldWithInlineComment(fieldName, resolvedValue, commentText);
    }
    toYaml() {
        const topLevelDescription = this.config.getConfigDescription();
        return this.toYamlSectionHeadingComment(topLevelDescription.overview) + '\n' +
            '\n' +
            this.toYamlFieldWithFieldDescription('config_root', this.config.getConfigRoot(), topLevelDescription.fieldDescriptions.config_root) + '\n' +
            '\n' +
            this.toYamlFieldWithFieldDescription('log_folder', this.config.getLogFolder(), topLevelDescription.fieldDescriptions.log_folder) + '\n' +
            '\n' +
            this.toYamlComment(topLevelDescription.fieldDescriptions.rules.descriptionText) + '\n' +
            this.toYamlRuleOverrides() + '\n' +
            '\n' +
            this.toYamlComment(topLevelDescription.fieldDescriptions.engines.descriptionText) + '\n' +
            this.toYamlEngineOverrides() + '\n' +
            '\n' +
            this.toYamlSectionHeadingComment((0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.common.end-of-config')) + '\n';
    }
    toYamlRuleOverrides() {
        if (this.userRules.getCount() === 0) {
            const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.yaml.no-rules-selected');
            return `rules: {} ${this.toYamlComment(commentText)}`;
        }
        let yamlCode = 'rules:\n';
        for (const engineName of this.userRules.getEngineNames()) {
            yamlCode += '\n';
            yamlCode += (0, StylingUtil_1.indent)(this.toYamlRuleOverridesForEngine(engineName), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
    toYamlRuleOverridesForEngine(engineName) {
        const engineConfigHeader = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.rule-overrides-section', [engineName.toUpperCase()]);
        let yamlCode = this.toYamlSectionHeadingComment(engineConfigHeader) + '\n';
        yamlCode += `${engineName}:\n`;
        for (const userRule of this.userRules.getRulesFor(engineName)) {
            const defaultRule = this.getDefaultRuleFor(engineName, userRule.getName());
            yamlCode += (0, StylingUtil_1.indent)(this.toYamlRuleOverridesForRule(userRule, defaultRule), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
    getDefaultRuleFor(engineName, ruleName) {
        try {
            return this.allDefaultRules.getRule(engineName, ruleName);
        }
        catch (e) {
            // istanbul ignore next
            return null;
        }
    }
    toYamlRuleOverridesForRule(userRule, defaultRule) {
        const userSeverity = userRule.getSeverityLevel();
        const userTags = userRule.getTags();
        return `"${userRule.getName()}":\n` +
            (0, StylingUtil_1.indent)(this.toYamlField('severity', userSeverity, defaultRule !== null ? defaultRule.getSeverityLevel() : userSeverity), 2) + '\n' +
            (0, StylingUtil_1.indent)(this.toYamlField('tags', userTags, defaultRule !== null ? defaultRule.getTags() : userTags), 2);
    }
    toYamlEngineOverrides() {
        if (this.relevantEngines.size === 0) {
            const commentText = (0, messages_1.getMessage)(messages_1.BundleName.ConfigModel, 'template.yaml.no-engines-selected');
            return `engines: {} ${this.toYamlComment(commentText)}`;
        }
        let yamlCode = 'engines:\n';
        for (const engineName of this.relevantEngines.keys()) {
            yamlCode += (0, StylingUtil_1.indent)(this.toYamlEngineOverridesForEngine(engineName), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
    toYamlEngineOverridesForEngine(engineName) {
        const engineConfigDescriptor = this.codeAnalyzer.getEngineConfigDescription(engineName);
        const userEngineConfig = this.codeAnalyzer.getEngineConfig(engineName);
        let yamlCode = '\n' +
            this.toYamlSectionHeadingComment(engineConfigDescriptor.overview) + '\n' +
            `${engineName}:\n`;
        // By fiat, the field description will always include, at minimum, an entry for "disable_engine", so we can
        // assume that the object is not undefined.
        for (const configField of Object.keys(engineConfigDescriptor.fieldDescriptions)) {
            const fieldDescription = engineConfigDescriptor.fieldDescriptions[configField];
            const resolvedValue = userEngineConfig[configField] ?? fieldDescription.defaultValue;
            // Add a leading newline to visually break up the property from the previous one.
            yamlCode += '\n' +
                (0, StylingUtil_1.indent)(this.toYamlFieldWithFieldDescription(configField, resolvedValue, fieldDescription), 2) + '\n';
        }
        return yamlCode.trimEnd();
    }
}
class PlainYamlFormatter extends YamlFormatter {
    constructor(codeAnalyzer, userRules, allDefaultRules, relevantEngines) {
        super(codeAnalyzer, userRules, allDefaultRules, relevantEngines);
    }
    toYamlComment(commentText) {
        return commentText.replace(/^.*/gm, s => `# ${s}`);
    }
}
class StyledYamlFormatter extends YamlFormatter {
    constructor(codeAnalyzer, userRules, allDefaultRules, relevantEngines) {
        super(codeAnalyzer, userRules, allDefaultRules, relevantEngines);
    }
    toYamlComment(commentText) {
        return commentText.replace(/^.*/gm, s => (0, StylingUtil_1.makeGrey)(`# ${s}`));
    }
}
function replaceAbsolutePathsWithRelativePathsWherePossible(value, parentFolder) {
    if (typeof value === 'string') {
        // Check if the string starts with the parent folder
        if (value.startsWith(parentFolder)) {
            // Strip the parent folder from the start of the string
            return value.substring(parentFolder.length);
        }
        return value; // Return unchanged if it doesn't start with the parent folder
    }
    else if (Array.isArray(value)) {
        // If value is an array, recursively process each element
        return value.map(item => replaceAbsolutePathsWithRelativePathsWherePossible(item, parentFolder));
    }
    else if (typeof value === 'object' && value !== null) {
        // If value is an object, recursively process each key-value pair
        const updatedObject = {};
        for (const key in value) {
            updatedObject[key] = replaceAbsolutePathsWithRelativePathsWherePossible(value[key], parentFolder);
        }
        return updatedObject;
    }
    // Return the value unchanged if it's a number, boolean, or null
    return value;
}
//# sourceMappingURL=ConfigModel.js.map