"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResultsTableDisplayer = exports.ResultsDetailDisplayer = exports.ResultsNoOpDisplayer = void 0;
exports.findLongestCommonParentFolderOf = findLongestCommonParentFolderOf;
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const StylingUtil_1 = require("../utils/StylingUtil");
const messages_1 = require("../messages");
const node_path_1 = __importDefault(require("node:path"));
class ResultsNoOpDisplayer {
    view(_results) {
        // istanbul ignore next - No need to cover deliberate no-op
        return;
    }
}
exports.ResultsNoOpDisplayer = ResultsNoOpDisplayer;
class AbstractResultsDisplayer {
    display;
    constructor(display) {
        this.display = display;
    }
    view(results) {
        if (results.getViolationCount() === 0) {
            return;
        }
        else {
            this.displayLineSeparator();
            this._view(results);
            this.displayLineSeparator();
        }
    }
    displayLineSeparator() {
        this.display.displayLog('');
    }
}
class ResultsDetailDisplayer extends AbstractResultsDisplayer {
    _view(results) {
        const violations = sortViolations(results.getViolations());
        const runDir = results.getRunDirectory();
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.shared.results-relative-to', [runDir]) + "\n");
        this.displayDetails(violations, runDir);
    }
    displayDetails(violations, runDir) {
        const styledViolations = violations
            .map((violation, idx) => this.styleViolation(violation, idx, runDir));
        this.display.displayLog(styledViolations.join('\n\n'));
    }
    styleViolation(violation, idx, runDir) {
        const rule = violation.getRule();
        const sev = rule.getSeverityLevel();
        const header = (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.detail.violation-header', [idx + 1, rule.getName()]);
        const body = {
            severity: `${sev.valueOf()} (${code_analyzer_core_1.SeverityLevel[sev]})`,
            engine: rule.getEngineName(),
            message: violation.getMessage()
        };
        const keys = ['severity', 'engine', 'message'];
        if (violation.getCodeLocations().length == 1) {
            body['location'] = stringifyLocation(violation.getCodeLocations()[0], false, runDir);
            keys.push('location');
        }
        else if (violation.getCodeLocations().length > 1) {
            body['locations'] = stringifyLocations(violation.getCodeLocations(), violation.getPrimaryLocationIndex(), runDir);
            keys.push('locations');
        }
        if (violation.getResourceUrls().length == 1) {
            body['resource'] = violation.getResourceUrls()[0];
            keys.push('resource');
        }
        else if (violation.getResourceUrls().length > 1) {
            body['resources'] = violation.getResourceUrls();
            keys.push('resources');
        }
        return (0, StylingUtil_1.toStyledHeaderAndBody)(header, body, keys);
    }
}
exports.ResultsDetailDisplayer = ResultsDetailDisplayer;
function stringifyLocations(codeLocations, primaryIndex, runDir) {
    return codeLocations.map((loc, idx) => stringifyLocation(loc, codeLocations.length > 1 && primaryIndex === idx, runDir));
}
function stringifyLocation(loc, displayMain, runDir) {
    const mainPortion = displayMain ? '(main) ' : '';
    let filePortion = loc.getFile();
    if (filePortion && filePortion.startsWith(runDir)) {
        filePortion = filePortion.slice(runDir.length);
    }
    let rangePortion = '';
    if (loc.getStartLine()) {
        rangePortion += ` (${loc.getStartLine()}:${loc.getStartColumn() || 1}`;
        if (loc.getEndLine()) {
            rangePortion += `-${loc.getEndLine()}:${loc.getEndColumn() || 1}`;
        }
        rangePortion += ')';
    }
    const commentPortion = loc.getComment() ? ` "${loc.getComment()}"` : '';
    return `${mainPortion}${filePortion}${rangePortion}${commentPortion}`;
}
const TABLE_COLUMNS = {
    num: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.num-column'),
    },
    severity: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.severity-column')
    },
    rule: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.rule-column')
    },
    location: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.location-column')
    },
    message: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.message-column')
    }
};
class ResultsTableDisplayer extends AbstractResultsDisplayer {
    _view(results) {
        const violations = sortViolations(results.getViolations());
        const parentFolder = findLongestCommonParentFolderOf(violations.map(v => v.getPrimaryLocation().getFile()).filter(f => f !== undefined));
        const resultRows = violations.map((v, idx) => {
            const severity = v.getRule().getSeverityLevel();
            const primaryLocation = v.getPrimaryLocation();
            const relativeFile = primaryLocation.getFile() ?
                node_path_1.default.relative(parentFolder, primaryLocation.getFile()) : null;
            return {
                num: idx + 1,
                location: `${relativeFile}:${primaryLocation.getStartLine()}:${primaryLocation.getStartColumn()}`,
                rule: `${v.getRule().getEngineName()}:${v.getRule().getName()}`,
                severity: `${severity.valueOf()} (${code_analyzer_core_1.SeverityLevel[severity]})`,
                message: v.getMessage()
            };
        });
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.shared.results-relative-to', [parentFolder]));
        this.display.displayTable(resultRows, TABLE_COLUMNS);
    }
}
exports.ResultsTableDisplayer = ResultsTableDisplayer;
function sortViolations(violations) {
    return violations.toSorted((v1, v2) => {
        // First compare severities.
        const v1Sev = v1.getRule().getSeverityLevel();
        const v2Sev = v2.getRule().getSeverityLevel();
        if (v1Sev !== v2Sev) {
            return v1Sev - v2Sev;
        }
        // Next, compare file names.
        const v1PrimaryLocation = v1.getCodeLocations()[v1.getPrimaryLocationIndex()];
        const v2PrimaryLocation = v2.getCodeLocations()[v2.getPrimaryLocationIndex()];
        const v1File = v1PrimaryLocation.getFile() || /* istanbul ignore next */ '';
        const v2File = v2PrimaryLocation.getFile() || /* istanbul ignore next */ '';
        if (v1File !== v2File) {
            return v1File.localeCompare(v2File);
        }
        // Next, compare start lines.
        const v1StartLine = v1PrimaryLocation.getStartLine() || /* istanbul ignore next */ 0;
        const v2StartLine = v2PrimaryLocation.getStartLine() || /* istanbul ignore next */ 0;
        if (v1StartLine !== v2StartLine) {
            return v1StartLine - v2StartLine;
        }
        // Next, compare start columns.
        const v1StartColumn = v1PrimaryLocation.getStartColumn() || /* istanbul ignore next */ 0;
        const v2StartColumn = v2PrimaryLocation.getStartColumn() || /* istanbul ignore next */ 0;
        return v1StartColumn - v2StartColumn;
    });
}
/**
 * Returns the longest comment parent folder of the file paths provided
 * Note that this function assumes that all the paths are indeed files and not folders
 */
function findLongestCommonParentFolderOf(filePaths) {
    const roots = filePaths.map(filePath => node_path_1.default.parse(filePath).root);
    const commonRoot = (new Set(roots)).size === 1 ? roots[0] : '';
    // istanbul ignore next - Hard to test outside of Windows
    if (!commonRoot) {
        return '';
    }
    const commonFolders = [];
    let depth = 0;
    const explodedPaths = filePaths.map(file => node_path_1.default.dirname(file).slice(commonRoot.length).split(node_path_1.default.sep).filter(v => v.length > 0));
    while (explodedPaths.every(folders => folders.length > depth)) {
        const currFolder = explodedPaths[0][depth];
        if (explodedPaths.some(folders => folders[depth] !== currFolder)) {
            break;
        }
        commonFolders.push(currFolder);
        depth++;
    }
    return node_path_1.default.join(commonRoot, ...commonFolders);
}
//# sourceMappingURL=ResultsViewer.js.map